/* GDK - The GIMP Drawing Kit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
 */

#include "config.h"

#include "gdkvisual.h"
#include "gdkprivate-directfb.h"


static void  gdk_visual_decompose_mask (gulong     mask,
                                        gint      *shift,
                                        gint      *prec);

static GdkVisualDirectFB *system_visual;

static gint available_depths[1];

static GdkVisualType available_types[1];

void
gdk_visual_init (void)
{
  system_visual = g_new (GdkVisualDirectFB, 1);

  system_visual->base.type = GDK_VISUAL_TRUE_COLOR;
  system_visual->base.red_mask   = 0x0000F800;
  system_visual->base.green_mask = 0x000007E0;
  system_visual->base.blue_mask  = 0x0000001F;

  system_visual->base.depth = 16;
  system_visual->base.byte_order = GDK_LSB_FIRST;
  system_visual->base.bits_per_rgb = 8;

  if ((system_visual->base.type == GDK_VISUAL_TRUE_COLOR) ||
      (system_visual->base.type == GDK_VISUAL_DIRECT_COLOR))
    {
      gdk_visual_decompose_mask (   system_visual->base.red_mask,
                                    &system_visual->base.red_shift,
                                    &system_visual->base.red_prec);

      gdk_visual_decompose_mask (   system_visual->base.green_mask,
                                    &system_visual->base.green_shift,
                                    &system_visual->base.green_prec);

      gdk_visual_decompose_mask (   system_visual->base.blue_mask,
                                    &system_visual->base.blue_shift,
                                    &system_visual->base.blue_prec);
    }
  else {
    system_visual->base.red_mask = 0;
    system_visual->base.red_shift = 0;
    system_visual->base.red_prec = 0;
    
    system_visual->base.green_mask = 0;
    system_visual->base.green_shift = 0;
    system_visual->base.green_prec = 0;
    
    system_visual->base.blue_mask = 0;
    system_visual->base.blue_shift = 0;
    system_visual->base.blue_prec = 0;
  }

  system_visual->base.colormap_size = 65536;   //FIXME
  
  available_depths[0] = system_visual->base.depth;
  available_types[0] = system_visual->base.type;
}

GdkVisual*
gdk_visual_ref (GdkVisual *visual)
{
  return visual;
}

void
gdk_visual_unref (GdkVisual *visual)
{
  return;
}

gint
gdk_visual_get_best_depth (void)
{
  return available_depths[0];
}

GdkVisualType
gdk_visual_get_best_type (void)
{
  return available_types[0];
}

GdkVisual*
gdk_visual_get_system (void)
{
  return ((GdkVisual*) system_visual);
}

GdkVisual*
gdk_visual_get_best (void)
{
  return ((GdkVisual*) system_visual);
}

GdkVisual*
gdk_visual_get_best_with_depth (gint depth)
{
  if (depth == system_visual->base.depth)
    return (GdkVisual*) system_visual;
  else
    return NULL;
}

GdkVisual*
gdk_visual_get_best_with_type (GdkVisualType visual_type)
{
  if (visual_type == system_visual->base.type)
    return (GdkVisual*) system_visual;
  else
    return NULL;
}

GdkVisual*
gdk_visual_get_best_with_both (gint          depth,
                               GdkVisualType visual_type)
{
  if ((depth == system_visual->base.depth) && (visual_type == system_visual->base.type))
    return (GdkVisual*) system_visual;
  else
    return NULL;
}

void
gdk_query_depths  (gint **depths,
                   gint  *count)
{
  *count = 1;
  *depths = available_depths;
}

void
gdk_query_visual_types (GdkVisualType **visual_types,
                        gint           *count)
{
  *count = 1;
  *visual_types = available_types;
}

GList*
gdk_list_visuals (void)
{
  return g_list_append (NULL, (gpointer) system_visual);
}

static void
gdk_visual_decompose_mask (gulong  mask,
                           gint   *shift,
                           gint   *prec)
{
  *shift = 0;
  *prec = 0;

  while (!(mask & 0x1)) {
    (*shift)++;
    mask >>= 1;
  }

  while (mask & 0x1) {
    (*prec)++;
    mask >>= 1;
  }
}

